import datetime
import os

import openpyxl
import xlrd


class SpecialExcel():

    def find_excel_data_change_datatype(self, row_data: list):
        """
        excel单元格的内容类型检测和转换
        参数：
            row_data：行数据，列表格式
        """
        result_data = []
        for rd in row_data:
            if type(rd) == datetime.datetime:
                t = rd.strftime("%Y-%m-%d %H:%M:%S")
            elif type(rd) == str:
                t = rd
            elif type(rd) == int:
                t = str(rd)
            elif type(rd) == float:
                t = str(rd)
            elif type(rd) is None:
                t = ''
            else:
                t = str(rd)
            result_data.append(t)
        return result_data

    def find_excel_data_find_key(self, search_key: str, row_content: str):
        """
        检测关键词和内容
        参数：
            search_key：关键词
            row_content：行内容
        """
        if search_key in row_content:
            return True
        else:
            return False

    def find_excel_data_process_xls(self, path, file):
        """
        读取xls后缀的excel文件
        参数：
            path：文件所在路径
            file：文件名
        """
        filepath = os.path.join(path, file)
        try:
            rb = xlrd.open_workbook(filepath, formatting_info=True)
        except:
            return False
        sheet_names = rb.sheet_names()
        space_line = 0
        for ws_name in sheet_names:
            ws = rb.sheet_by_name(ws_name)
            rows = ws.nrows
            cols = ws.ncols
            for r in range(rows):
                values = [ws.cell(r, c).value for c in range(cols)]
                values = self.find_excel_data_change_datatype(values)
                values = " ".join(values)
                if values:
                    yield filepath, ws_name, r, values  # 文件路径，工作表名，行数，行内容
                else:
                    if space_line < 10:
                        space_line += 1
                    else:
                        break

    def find_excel_data_process_xlsx(self, path, file):
        """
        读取xlsx后缀的excel文件
        参数：
            path：文件所在路径
            file：文件名
        """
        filepath = os.path.join(path, file)
        try:
            wb = openpyxl.load_workbook(filepath, read_only=True, data_only=True)
        except:
            return False
        worksheets_name = wb.sheetnames
        space_line = 0
        for ws_name in worksheets_name:
            ws = wb[ws_name]
            for index, row in enumerate(ws.rows):
                values = [r.value for r in row if r.value != None]
                values = self.find_excel_data_change_datatype(values)
                values = " ".join(values)
                if values:
                    yield filepath, ws_name, index, values  # 文件路径，工作表名，行数，行内容
                else:
                    if space_line < 10:
                        space_line += 1
                    else:
                        break

    def find_excel_data_main(self, search_key, target_dir):
        """
        检索指定目录下的excel文件和过滤
        参数：
            search_key：检索的关键词
            target_dir：目标文件夹
        """
        for path, dirs, files in os.walk(target_dir):
            files = [file for file in files if not file.startswith('~$')]  # 过滤掉正打开的excel文件
            xls_files = [file for file in files if file.endswith('.xls')]  # 取出所有的xls后缀文件
            xlsx_files = [file for file in files if file.endswith('.xlsx')]  # 取出所有的xlsx后缀文件
            for xls in xls_files:
                for data in self.find_excel_data_process_xls(path, xls):
                    filepath, ws_name, index, values = data
                    status = self.find_excel_data_find_key(search_key, values)
                    if status:
                        yield filepath, ws_name, index, values
            for xlsx in xlsx_files:
                for data in self.find_excel_data_process_xlsx(path, xlsx):
                    filepath, ws_name, index, values = data
                    status = self.find_excel_data_find_key(search_key, values)
                    if status:
                        yield filepath, ws_name, index, values  # 输出内容：路径/文件名、工作表名、行数、行内容

    def find_excel_data(self, search_key, target_dir):
        for data in self.find_excel_data_main(search_key, target_dir):
            print(data)


from search4file.core.SearchByContent import SearchByContent
from search4file.core.SpecialFunc import SpecialExcel
from search4file.lib.utils.except_utils import except_dec

sbc = SearchByContent()

se = SpecialExcel()


# @except_dec()
def search_by_content(search_path, search_content):
    search_result_dict = sbc.search_files(search_path, search_content)
    print(search_result_dict)
    return search_result_dict


# @except_dec()
def find_excel_data(search_key, target_dir):
    se.find_excel_data(search_key, target_dir)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: 程序员晚枫
# Mail: 1957875073@qq.com
# Created Time:  2022-4-19 10:17:34
# Description: https://mp.weixin.qq.com/s/zzD4pxNMFd0ZuWqXlVFdAg
#############################################

from setuptools import setup            #这个包没有的可以pip一下

setup()



import unittest

from search4file import __version__
from search4file.api.search_by_content import search_by_content, find_excel_data


class TestSearch(unittest.TestCase):
    def test_version(self):
        assert __version__ == '0.1.0'

    def test_search_by_content(self):
        search_by_content(search_path=r'D:/download', search_content='程序员晚枫')

    def test_find_excel_data(self):
        find_excel_data(search_key='刘家站垦殖场',
                        target_dir=r'D:\workplace\code\gitee\python-office\contributors\bulabean')



import glob
import json
import os

import fitz
import pandas as pd
from docx import Document
from pptx import Presentation

from search4file.core import SpecialExcel


class SearchByContent():
    # 初始化输出结果
    def __init__(self):
        # 查找结果，返回格式，3部分
        self.search_result_dict = {
            'search_path': None,
            'search_content': None,
            # 初始化、找不到时：
            'search_result': {}
        }
        # 各类文件的查找结果
        self.files_result_dict = {}
        self.word_result_dict = {}
        self.excel_result_dict = {}
        self.ppt_result_dict = {}
        self.pdf_result_dict = {}

    # 格式化返回内容
    def create_return_result(self, search_path, search_content):
        # 查找路径，转为绝对路径
        search_path = os.path.abspath(search_path)
        # 查询路径在哪里
        self.search_result_dict['search_path'] = search_path
        # 查询内容是什么
        self.search_result_dict['search_content'] = search_content
        # 返回纯文本的结果
        if len(self.files_result_dict) > 0:
            self.search_result_dict['search_result']['files'] = self.files_result_dict
        # 返回word的结果
        if len(self.word_result_dict) > 0:
            self.search_result_dict['search_result']['word'] = self.word_result_dict
        # 返回excel的结果
        if len(self.excel_result_dict) > 0:
            self.search_result_dict['search_result']['excel'] = self.excel_result_dict
        # 返回 pdf 的结果
        if len(self.pdf_result_dict) > 0:
            self.search_result_dict['search_result']['pdf'] = self.pdf_result_dict
        # 返回 ppt 的结果
        if len(self.ppt_result_dict) > 0:
            self.search_result_dict['search_result']['ppt'] = self.ppt_result_dict
        # 转换为json格式返回
        return json.dumps(self.search_result_dict, indent=4, ensure_ascii=False)

    # 搜索内容的逻辑
    def search_files(self, search_path, search_content):
        glob_path = glob.glob(search_path)  # 获取全部路径
        for file_path in glob_path:  # for 循环判断递归查到的内容是文件夹还是文件
            if glob.os.path.isdir(file_path):  # 若是文件夹，继续将该文件夹的路径传给 search() 函数继续递归查找
                _path = glob.os.path.join(file_path, '*')
                self.search_files(_path, search_content)
            else:
                if file_path.endswith("docx"):  # 搜索word文件
                    self.search_word_file(file_path, search_content)
                elif file_path.endswith("xlsx") or file_path.endswith("xls"):  # 搜索excel文件
                    self.search_excel_file(file_path, search_content)
                elif file_path.endswith('pdf'):  # 搜索pdf文件
                    self.search_pdf_file(file_path, search_content)
                elif file_path.endswith('pptx'):  # 搜索pdf文件
                    self.search_ppt_file(file_path, search_content)
                else:
                    self.search_txt_file(file_path, search_content)  # 没有任何匹配后缀，搜索纯文本文件

        return self.create_return_result(search_path, search_content)

    ####################################
    # 搜索 纯文本 文件
    ####################################
    def search_txt_file(self, file_path, search_content) -> None:
        """
        :param search_path:
        :param content:
        :return: dict
        """
        # 利用 open() 函数读取文件，并通过 try...except... 捕获不可读的文件格式（.zip 格式）
        try:
            with open(file_path, 'r', encoding='utf-8') as file:
                if search_content in file.read():
                    # 若是文件，则将该查询到的文件所在路径插入 final_result 空列表
                    self.files_result_dict[str(len(self.files_result_dict) + 1)] = file_path
        except:
            pass

    ####################################
    # 搜索 word 文件
    ####################################
    def search_word_file(self, file_path, search_content):

        document = Document(file_path)
        all_paragraphs = document.paragraphs
        for paragraph in all_paragraphs:
            if paragraph.text.find(search_content) >= 0:
                self.word_result_dict[str(len(self.word_result_dict) + 1)] = file_path
                break

    ####################################
    # 搜索 excel 文件
    ####################################
    def search_excel_file(self, file_path, search_content):
        df = pd.read_excel(file_path, sheet_name=None, header=None)
        for sheet in df.values():
            for row in sheet.itertuples():
                if search_content in row:
                    self.excel_result_dict[str(len(self.excel_result_dict) + 1)] = file_path
                    break

    ####################################
    # 搜索 pdf 文件
    ####################################
    def search_pdf_file(self, file_path, search_content):
        for page in fitz.open(file_path):  # iterate the document pages
            if page.search_for(search_content):
                self.pdf_result_dict[str(len(self.pdf_result_dict) + 1)] = file_path
                break

    ####################################
    # 搜索 ppt 文件
    ####################################
    def search_ppt_file(self, file_path, search_content):
        ppt = Presentation(file_path)
        for slide in ppt.slides:  # > .slides 得到一个列表，包含每个列表slide
            for shape in slide.shapes:  # > slide.shapes 形状
                if shape.has_text_frame:  # shape.has_text_frame 判断是否有文字
                    # text_frame = shape.text_frame  # shape.text_frame 获取文字框
                    for paragraph in shape.text_frame.paragraphs:  # text_frame.paragraphs 获取段落
                        if search_content in paragraph.text:
                            self.ppt_result_dict[str(len(self.ppt_result_dict) + 1)] = file_path
                            break
